import os
import sys
from collections import Mapping
from copy import deepcopy
from datetime import datetime
from logging import Formatter

try:
    import simplejson as json
except ImportError:
    import json


JSON_DEFAULT_LOG_KEYS = ['threadName', 'process', 'args', 'module', 'levelname', 'pathname', 'lineno', 'funcName']
JSON_LOG_KEYS = os.getenv('SPS_JSON_LOG_KEYS', JSON_DEFAULT_LOG_KEYS)
if not isinstance(JSON_LOG_KEYS, list):
    try:
        JSON_LOG_KEYS = JSON_LOG_KEYS.split(',')
    except:
        JSON_LOG_KEYS = JSON_DEFAULT_LOG_KEYS


class BaseLogFormatter(Formatter):

    @classmethod
    def format_timestamp(cls, time):
        tstamp = datetime.utcfromtimestamp(time)
        return tstamp.strftime("%Y-%m-%dT%H:%M:%S") + ".%03d" % (tstamp.microsecond / 1000) + "Z"

    @classmethod
    def serialize(cls, message):
        if sys.version_info < (3, 0):
            return json.dumps(message)
        return bytes(json.dumps(message), 'utf-8')


class JSONLogFormatter(BaseLogFormatter):

    def format(self, record):
        logmsg = deepcopy(record.__dict__.get('msg'))
        if not isinstance(logmsg, Mapping):
            logmsg = {'message': logmsg}

        meta = {}
        for key in record.__dict__.keys():
            if key in JSON_LOG_KEYS:
                meta[key] = record.__dict__.get(key)
        logmsg['__meta'] = meta

        title = '%s.%s:%d' % (record.__dict__.get('module'), record.__dict__.get('funcName'), record.__dict__.get('lineno'))
        div = '==={0:{fill}<50}'.format(title, fill='=')

        try:
            msg = self.dumps(logmsg)
        except TypeError:
            msg = logmsg

        record.msg = "{div}\n{msg}\n{hr}".format(
            div='\033[1;30m{}\033[0m'.format(div),
            msg=msg,
            hr='\033[1;30m{}\033[0m'.format('.' * 53)
        )
        return super(JSONLogFormatter, self).format(record)

    def dumps(self, logmsg):
        return json.dumps(logmsg, indent=4)


class JSONFlatLogFormatter(JSONLogFormatter):

    def dumps(self, logmsg):
        return json.dumps(logmsg)



import logging
import logging.config
import os

LOG_LOGGER = os.getenv('SPY_LOG_LOGGER', 'json-flat')
LOG_LEVEL = os.getenv('SPY_LOG_LEVEL', 'ERROR')

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'json': {'()': 'spylogger.formatters.JSONLogFormatter'},
        'json-flat': {'()': 'spylogger.formatters.JSONFlatLogFormatter'},
        "ugly": {"format": "%(asctime)s %(levelname)s: %(module)s.%(funcName)s:%(lineno)d  [%(message)s]"}
    },
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler',
            'level': 'DEBUG',
            'formatter': 'json',
            'stream': 'ext://sys.stdout'
        },
        'cw': {
            'class': 'logging.StreamHandler',
            'level': 'DEBUG',
            'formatter': 'json-flat',
            'stream': 'ext://sys.stdout'
        },
        'ugly': {
            'class': 'logging.StreamHandler',
            'level': 'DEBUG',
            'formatter': 'ugly',
            'stream': 'ext://sys.stdout'
        },
    },
    'loggers': {
        'json-flat': {'level': LOG_LEVEL, 'handlers': ['cw']},
        'json': {'level': LOG_LEVEL, 'handlers': ['console']},
        'ugly': {'level': LOG_LEVEL, 'handlers': ['ugly']},
    }
}


def get_logger(name=LOG_LOGGER, log_level=LOG_LEVEL):
    logging.config.dictConfig(LOGGING)
    logger = logging.getLogger(name)
    logger.setLevel(log_level)
    return logger



from pip.req import parse_requirements
from pip.download import PipSession
from setuptools import setup

INSTALL_REQS = parse_requirements('requirements.pip', session=PipSession())

setup(
    name='spylogger',
    packages=['spylogger'],
    version='1.0.3',
    include_package_data=True,
    description='Python logging library',
    long_description=open('README.rst').read(),
    url='https://github.com/SPSCommerce/spylogger',
    author='meganlkm',
    author_email='webapps@spscommerce.com',
    install_requires=[str(ir.req) for ir in INSTALL_REQS],
    keywords=['logging'],
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: System :: Logging',
    ]
)
